#region Using declarations
using System;
using System.Collections;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// TD Sequential/Combo Countdown
    /// </summary>
    [Description("TD Sequential/Combo Countdown")]
    public class IGTDSequential : Indicator
    {
        #region Variables
        // Wizard generated variables
        private int _intSetupLookBackBars = 4; // Default setting for SetupLookBackBars
        private int _intSetupBars = 9; // Default setting for SetupBars
		
        // User defined variables (add any user defined variables below)
		private ArrayList _alTDSTHelper;
		
		private bool _boolBearishTDPriceFlipOccurred = false;
		private bool _boolBullishTDPriceFlipOccurred = false;
		private bool _boolComboCountownInProgress = false;
		private bool _boolContinueSetupCountDown = true;
		private bool _boolDrawBar1Arrow = true;
		private bool _boolDrawBar13Arrow = true;
		private bool _boolDrawPerfectSignalArrow = true;
		private bool _boolDrawPriceFlipArrow = true;
		private bool _boolPlotComboCountDown = true;
		private bool _boolPlotSequentialCountDown = true;
		private bool _boolSequentialCountownInProgress = false;
		private bool _boolSetupTDSTOn = true;
		private bool _boolSequentialTDSTOn = false;
		
		private Color _colorComboFontColor = Color.White;
		private Color _colorComboFontColorRecycle = Color.White;
		private Color _colorSequentialFontColor = Color.Gray;
		private Color _colorSequentialFontColorRecycle = Color.Black;
		private Color _colorSetupFontColorBar1 = Color.Red;
		private Color _colorSetupFontColorBar9 = Color.Black;
		private Color _colorSetupFontColor = Color.Black;
		private Color _colorSetupFontColorPriceFlip = Color.Black;
		private Color _colorSetupTDSTBuy = Color.Green;
		private Color _colorSetupTDSTSell = Color.Red;
		
		private DashStyle _dsTDST = DashStyle.Dash;
		
		private DataSeries _dsYLocation;
		
		private double _doubleLastComboClose = 0;
		private double _doubleLastComboHigh = 0;
		private double _doubleLastComboLow = 0;
		private double _doubleMult = 0;
		private double _doublePointOffset = 0; 
		private double _doubleSequentialCountdownBar8Close = 0; 
		private double _doubleSetupCountdownBar1High = 0; 
		private double _doubleSetupCountdownBar1Low = 0; 
		private double _doubleYLocation = 0;
		
		private int _intArrowCount = 0;
		private int _intComboCount = 0;
		private int _intComboFontSize = 10;
		private int _intComboFontSizeRecycle = 13;
		private int _intObjectCount = 0;
		private int _intSequentialCount = 0;
		private int _intSequentialFontSize = 10;
		private int _intSequentialFontSizeRecycle = 13;
		private int _intSequentialRecycleCount = 0;
		private int _intSequentialTags = 0;
		private int _intSetupCount = 0;
		private int _intSetupFontSize = 10;
		private int _intSetupTDSTLength = 55;
		private int _intSetupTDSTMaxNumber = 3;
		private int _intSetupTDSTWidth = 2;
		
		private string _strSetupBar1Arrow = string.Empty;
		private string[] _strComboTags;
		private string[] _strSequentialTags;
		private string[] _strSetupTags;
		
		private TDSequentialPlotType _TDSequentialPlotType = TDSequentialPlotType.DrawText;
		private TDType _TDCountdownType = TDType.TDBuy;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            this.CalculateOnBarClose = true;
            this.Overlay = true;
            this.PriceTypeSupported	= false;
			
			this._alTDSTHelper = new ArrayList();
			this._dsYLocation = new DataSeries(this);
			if (this.TDCountdownType == TDType.TDBuy)
			{
				this._doubleLastComboClose = double.MaxValue;
				this._doubleLastComboLow = double.MaxValue;
				this._doubleMult = -1;
			}
			else
			{
				this._doubleLastComboClose = double.MinValue;
				this._doubleLastComboHigh = double.MinValue;
				this._doubleMult = 1;
			}
			this._strComboTags = new String[377];
			this._strSequentialTags = new String[377];
			this._strSetupTags = new String[377];
        }

        /// <summary>
        /// Called on each bar update event (incoming tick).
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < this.SetupBars*2) return;
			this.HouseKeeping();
			if ((this.TDCountdownType == TDType.TDBuy && this.BearishTDPriceFlipOccurred == true) ||
				(this.TDCountdownType == TDType.TDSell && this.BullishTDPriceFlipOccurred == true))
				this.ProcessSetupCountDown();
			if (this.PlotSequentialCountDown &&	this.IsSetupCountDownCompleted == true && this.IsSequentialCountdownInProgress == false) this.IsSequentialCountdownInProgress = true;
			if (this.PlotSequentialCountDown &&	this.IsSequentialCountdownInProgress == true) this.ProcessSequentialCountDown();
			if (this.PlotComboCountDown &&	this.IsComboCountDownInProgress == true) this.ProcessComboCountDown();
			// TDST
			if (this.TDSTOn) this.ProcessTDST();
				
        }
		
		#region Private Methods
		
		#region HouseKeeping
		private void HouseKeeping()
		{
			if (this._doublePointOffset == 0) this._doublePointOffset = SMA(Range(), this.SetupBars)[0] * 0.15;
			if (this.TDCountdownType == TDType.TDBuy)
			{
				if (this.BearishTDPriceFlipOccurred == false) this.BearishTDPriceFlipOccurred = this.IsBearishFlip; // Look for a bearish td flip (page 3 of Bloomberg DeMark Indicators book)
				if (this.IsBullishFlip)
				{
					if (this.IsSetupCountdownInProgress) this.RemoveCurrentSetupCountdown();
					this.InitializeNewSetupCountdown();
					this.BearishTDPriceFlipOccurred = false;
				}
				if (Close[0] < Close[this.SetupLookBackBars]) this._intSequentialRecycleCount++; else this._intSequentialRecycleCount = 0;
				this._dsYLocation.Set(Low[0]);
				this._doubleYLocation = Low[0];
			}
			else
			{
				if (this.BullishTDPriceFlipOccurred == false) this.BullishTDPriceFlipOccurred = this.IsBullishFlip; // Look for a bullish td flip (page 4 of Bloomberg DeMark Indicators book)
				if (this.IsBearishFlip)
				{
					if (this.IsSetupCountdownInProgress) this.RemoveCurrentSetupCountdown();
					this.InitializeNewSetupCountdown();
					this.BullishTDPriceFlipOccurred = false;
				}
				if (Close[0] > Close[this.SetupLookBackBars]) this._intSequentialRecycleCount++; else this._intSequentialRecycleCount = 0;
				this._dsYLocation.Set(High[0]);
				this._doubleYLocation = High[0];
			}
			
			if (this.IsSetupCountDownCompleted) this.InitializeNewSetupCountdown();
		}
		#endregion
		
		#region Setup Countdown
		/// <summary>
		/// (page 9 of Bloomberg DeMark Indicators book)
		/// </summary>
		private void PerfectSignalTest()
		{
			if (this.TDCountdownType == TDType.TDBuy)
			{
				if ((Low[0] <= Low[2] && Low[0] <= Low[3]) || 
					(Low[1] <= Low[2] && Low[1] <= Low[3])) 
					if (this.DrawPerfectSignalArrow && this.DrawPerfectSignalArrow) DrawArrowUp(this._intArrowCount++.ToString(), false, 0, this._doubleYLocation - this._doublePointOffset * 3, Color.Black);
			}
			else
			{
				if ((High[0] >= High[2] && High[0] >= High[3]) || 
					(High[1] >= High[2] && High[1] >= High[3])) 
					if (this.DrawPerfectSignalArrow && this.DrawPerfectSignalArrow) DrawArrowDown(this._intArrowCount++.ToString(), false, 0, this._doubleYLocation + this._doublePointOffset * 3, Color.Black);
			}
		}
		
		/// <summary>
		/// 
		/// </summary>
		private void ProcessSetupCountDown()
		{
			if ((this.TDCountdownType == TDType.TDBuy && Close[0] < Close[this.SetupLookBackBars]) ||
				(this.TDCountdownType == TDType.TDSell && Close[0] > Close[this.SetupLookBackBars])) // Need 9 like this to complete a setup countdown
			{
				this._intSetupCount++;
				this._dsYLocation.Set(this._dsYLocation[0] + this._doublePointOffset * this._doubleMult);
				if (this._intSetupCount == 1) // Setup countdown bar 1
				{
					this.TDSetupDrawText(this._intSetupCount.ToString(), this._colorSetupFontColorBar1, this._intSetupFontSize);
					if (this.TDCountdownType == TDType.TDBuy && this.DrawBar1Arrow) DrawArrowDown(this._strSetupBar1Arrow = this._intArrowCount++.ToString(), false, 0, this._doubleYLocation - this._doublePointOffset * 5, Color.Red);
					if (this.TDCountdownType == TDType.TDSell && this.DrawBar1Arrow) DrawArrowUp(this._strSetupBar1Arrow = this._intArrowCount++.ToString(), false, 0, this._doubleYLocation + this._doublePointOffset * 5, Color.Lime);
					if (this.IsSequentialCountdownInProgress == false) 
					{
						this._doubleSetupCountdownBar1High = High[0];
						this._doubleSetupCountdownBar1Low = Low[0];
					}
				}
				else if (this.IsSetupCountDownCompleted) // Setup countdown bar 9
				{
					this.TDSetupDrawText(this._intSetupCount.ToString(), this._colorSetupFontColorBar9, this._intSetupFontSize);
					this.PerfectSignalTest();
					if (this.IsComboCountDownInProgress == false) this.StartComboCountDown();
					// TDST Helper
					if (this._alTDSTHelper.Count >= this.TDSTMaxNumber * 2)
					{
						this.RemoveDrawObject(((TDSTHelper)this._alTDSTHelper[0]).TDSTTag); // remove oldest line from chart
						this.RemoveDrawObject(((TDSTHelper)this._alTDSTHelper[1]).TDSTTag);
						this._alTDSTHelper.RemoveAt(0); // remove oldest line from array
						this._alTDSTHelper.RemoveAt(0);
					}
					int intBar = this.TDCountdownType == TDType.TDBuy ? HighestBar(High, 9) : LowestBar(Low, 9) + 0;
					double doubleY = this.TDCountdownType == TDType.TDBuy ? High[intBar] : Low[intBar];
					if (this.TDCountdownType == TDType.TDBuy && Close[9] > doubleY) // czpiaor
						doubleY = Close[9]; // czpiaor
					else if (this.TDCountdownType == TDType.TDSell && Close[9] < doubleY) // czpiaor
						doubleY = Close[9]; // czpiaor
					// TDST Bar 1
					TDSTHelper tdst1 = new TDSTHelper("TDSTBar1" + this.CurrentBar.ToString(), 
						this.CurrentBar - intBar, // start bar
						doubleY, // start Y
						this.CurrentBar - intBar + this.TDSTLength, // end bar
						doubleY, // end Y
						(this.TDCountdownType == TDType.TDBuy ? this.TDSTBuyColor : this.TDSTSellColor),
						this.TDSTDashStyle,
						this.TDSTWidth);
					this._alTDSTHelper.Add(tdst1);

					// ***** no longer used *****
					// ***** no longer used *****
					// ***** no longer used *****
					// ***** no longer used *****
					// ***** no longer used *****
					// TDST Bar 9 
					TDSTHelper tdst9 = new TDSTHelper("TDSTBar9" + this.CurrentBar.ToString(), 
						this.CurrentBar, // start bar
						(this.TDCountdownType == TDType.TDBuy ? Low[0] : High[0]), // start Y
						this.CurrentBar + this.TDSTLength, // end bar
						(this.TDCountdownType == TDType.TDBuy ? Low[0] : High[0]), // end Y
//						(this.TDCountdownType == TDType.TDBuy ? this.TDSTBuyColor : this.TDSTSellColor),
						Color.Transparent,
						this.TDSTDashStyle,
						this.TDSTWidth);
					this._alTDSTHelper.Add(tdst9);
					// ***** no longer used *****
					// ***** no longer used *****
					// ***** no longer used *****
					// ***** no longer used *****
					// ***** no longer used *****
				}
				else
				{
					if (this._TDSequentialPlotType == TDSequentialPlotType.DrawDots)
						this.TDSetupDrawDot();
					else
						this.TDSetupDrawText(this._intSetupCount.ToString(), this._colorSetupFontColor, this._intSetupFontSize);
				}
			}
			else
				this.InitializeNewSetupCountdown();
		}
		
		/// <summary>
		/// 
		/// </summary>
		private void RemoveCurrentSetupCountdown()
		{
			if (this._strSetupBar1Arrow.Length > 0) RemoveDrawObject(this._strSetupBar1Arrow);
			for (int int1 = 0; int1 < this._intSetupCount; int1++) RemoveDrawObject(this._strSetupTags[int1]);
		}

		/// <summary>
		/// 
		/// </summary>
		private void InitializeNewSetupCountdown()
		{
			if (this.IsSetupCountdownInProgress) this.RemoveCurrentSetupCountdown();
			if (this.IsSetupCountDownCompleted && (this.IsBearishFlip || this.IsBullishFlip))
			{
				this.TDSetupDrawText("PF", this._colorSetupFontColorPriceFlip, this._intSetupFontSize);
				if (this.TDCountdownType == TDType.TDBuy)
					if (this.DrawPriceFlipArrow) DrawArrowUp(this._intArrowCount++.ToString(), false, 0, this._doubleYLocation - this._doublePointOffset * 3, Color.Black);
				else
					if (this.DrawPriceFlipArrow) DrawArrowDown(this._intArrowCount++.ToString(), false, 0, this._doubleYLocation + this._doublePointOffset * 3, Color.Black);
			}
			this._strSetupBar1Arrow = string.Empty;
			this._strSetupTags.Initialize();
			this._intSetupCount = 0;
		}

		/// <summary>
		/// 
		/// </summary>
		private void TDSetupDrawDot()
		{
			this._intObjectCount++;
			if (this.TDCountdownType == TDType.TDBuy)
				DrawDot(this._intObjectCount.ToString(), true, 0, this._dsYLocation[0], Color.Lime);
			else
				DrawDot(this._intObjectCount.ToString(), true, 0, this._dsYLocation[0], Color.Red);
			this._strSetupTags.SetValue(this._intObjectCount.ToString(), this._intSetupCount-1); 
		}
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="strText"></param>
		/// <param name="colorColor"></param>
		/// <param name="intFontSize"></param>
		private void TDSetupDrawText(string strText, Color colorColor, int intFontSize)
		{
			this._intObjectCount++;
			DrawText(this._intObjectCount.ToString(), 
				false, 
				strText, 
				0, 
				this._dsYLocation[0], 
				colorColor, 
				new Font(FontFamily.GenericSansSerif, intFontSize, FontStyle.Bold, GraphicsUnit.Pixel), 
				StringAlignment.Center,
				Color.Transparent,
				Color.Transparent,
				0);
			this._strSetupTags.SetValue(this._intObjectCount.ToString(), _intSetupCount-1); 
		}
		#endregion
		
		#region Setup TDST
		private void ProcessTDST()
		{
			for (int index = 0; index < this._alTDSTHelper.Count; index++) 
			{
				if (((TDSTHelper)this._alTDSTHelper[index]).TDSTEndBar >= this.CurrentBar)
				{
					TDSTHelper tdst = (TDSTHelper)this._alTDSTHelper[index];
					this.DrawLine(tdst.TDSTTag, false, this.CurrentBar - tdst.TDSTStartBar, tdst.TDSTStartY, 0, tdst.TDSTEndY, tdst.TDSTColor, tdst.TDSTDashStyle, tdst.TDSTWidth);
				}
			}
		}
		#endregion
		
		#region Sequential Countdown
		/// <summary>
		/// 
		/// </summary>
		private void ProcessSequentialCountDown()
		{
			if ((this.TDCountdownType == TDType.TDBuy && Close[0] <= Low[2]) || // czpiaor
				(this.TDCountdownType == TDType.TDSell && Close[0] >= High[2])) // czpiaor
			{
				this._dsYLocation.Set(this._dsYLocation[0] + this._doublePointOffset * this._doubleMult);
				if (this._intSequentialCount < 12)
				{
					this._intSequentialCount++;
					this.TDSequentialDrawText(this._intSequentialCount.ToString(), this._colorSequentialFontColor, this._intSequentialFontSize);
					if (this._intSequentialCount == 8) this._doubleSequentialCountdownBar8Close = Close[0];
				}
				else if (this._intSequentialCount == 12) // additional requirement for bar 13 (page 19 of Bloomberg DeMark Indicators book)
				{
					if ((this.TDCountdownType == TDType.TDBuy && Low[0] <= this._doubleSequentialCountdownBar8Close) ||
						(this.TDCountdownType == TDType.TDSell && High[0] >= this._doubleSequentialCountdownBar8Close))
					{
						if (this._intSequentialRecycleCount >= 18) // (page 22 of Bloomberg DeMark Indicators book)
						{
							this.RecycleSequentialCountdown();
						}
						else
						{
							this._intSequentialCount++;
							this.TDSequentialDrawText(this._intSequentialCount.ToString(), this._colorSequentialFontColor, this._intSequentialFontSize);
							if (this.TDCountdownType == TDType.TDBuy && this.DrawBar13Arrow) DrawArrowUp(this._intArrowCount++.ToString(), false, 0, this._doubleYLocation - this._doublePointOffset * 1, Color.Lime);
							if (this.TDCountdownType == TDType.TDSell && this.DrawBar13Arrow) DrawArrowDown(this._intArrowCount++.ToString(), false, 0, this._doubleYLocation + this._doublePointOffset * 1, Color.Red);
							this.InitializeNewSequentialCountdown();
						}
					}
					else
					{
						this.TDSequentialDrawText("+", this._colorSequentialFontColor, this._intSequentialFontSize);
					}
				}
			}
			
			if (this.TDCountdownType == TDType.TDBuy)
			{				
				if (Low[0] > this._doubleSetupCountdownBar1High || this.IsBarTDSellSetup) // (page 21 of Bloomberg DeMark Indicators book)
				{
					this.RemoveCurrentSequentialCountdown();
					this.InitializeNewSequentialCountdown();
				}
			}				
			else
			{
				if (High[0] < this._doubleSetupCountdownBar1Low || this.IsBarTDBuySetup) // (page ?? of Bloomberg DeMark Indicators book)
				{
					this.RemoveCurrentSequentialCountdown();
					this.InitializeNewSequentialCountdown();
				}
			}				
		}
		
		/// <summary>
		/// 
		/// </summary>
		private void RemoveCurrentSequentialCountdown()
		{
			int int1 = 0;
			while (this._strSequentialTags[int1] != null)
			{
				RemoveDrawObject(this._strSequentialTags[int1]);
				int1++;
			}
		}
		
		/// <summary>
		/// 
		/// </summary>
		private void RecycleSequentialCountdown()
		{
			this.RemoveCurrentSequentialCountdown();
			this.TDSequentialDrawText("R", this._colorSequentialFontColorRecycle, this._intSequentialFontSizeRecycle);
			if (this.TDCountdownType == TDType.TDBuy)
				DrawArrowUp(this._intArrowCount++.ToString(), false, 0, this._doubleYLocation - this._doublePointOffset * 1, Color.Black);
			else
				DrawArrowDown(this._intArrowCount++.ToString(), false, 0, this._doubleYLocation + this._doublePointOffset * 1, Color.Black);
			this._intSequentialCount = 0;
			this._strSequentialTags.Initialize();
		}
		
		/// <summary>
		/// 
		/// </summary>
		private void InitializeNewSequentialCountdown()
		{
			this._intSequentialCount = 0;
			this._intSequentialTags = 0;
			for (int index = 0; index < this._strSequentialTags.Length; index++) 
				this._strSequentialTags.SetValue(null, index);
			this.IsSequentialCountdownInProgress = false;
		}
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="strText"></param>
		/// <param name="colorColor"></param>
		/// <param name="intFontSize"></param>
		private void TDSequentialDrawText(string strText, Color colorColor, int intFontSize)
		{
			this._intObjectCount++;
			DrawText(this._intObjectCount.ToString(), 
				false, 
				strText, 
				0, 
				this._dsYLocation[0], 
				colorColor, 
				new Font(FontFamily.GenericSansSerif, intFontSize, FontStyle.Bold, GraphicsUnit.Pixel), 
				StringAlignment.Center, 
				Color.Transparent, 
				Color.Transparent, 
				0); 
			this._strSequentialTags.SetValue(this._intObjectCount.ToString(), this._intSequentialTags++); 
		}
		#endregion
	
		#region Combo Countdown
		private void InitializeNewComboCountdown()
		{
			if (this.TDCountdownType == TDType.TDBuy)
			{
				this._doubleLastComboClose = double.MaxValue;
				this._doubleLastComboLow = double.MaxValue;
			}
			else
			{
				this._doubleLastComboClose = double.MinValue;
				this._doubleLastComboHigh = double.MinValue;
			}
			this._intComboCount = 0;
			this._strComboTags.Initialize();
			this.IsComboCountDownInProgress = false;
		}

		private void ProcessComboCountDown()
		{
			if ((this.TDCountdownType == TDType.TDBuy && (Close[0] <= Low[2] && Low[0] <= this._doubleLastComboLow && Close[0] < this._doubleLastComboClose && Close[0] < Close[1])) ||
				(this.TDCountdownType == TDType.TDSell && (Close[0] >= High[2] && High[0] >= this._doubleLastComboHigh && Close[0] > this._doubleLastComboClose && Close[0] > Close[1])))
			{
				this._dsYLocation.Set(this._dsYLocation[0] + this._doublePointOffset * this._doubleMult);
				this._intComboCount++;
				this.TDComboDrawText(this._intComboCount.ToString(), 0, this._dsYLocation[0], this._colorComboFontColor, this._intComboFontSize);
				this._doubleLastComboClose = Close[0];
				this._doubleLastComboHigh = High[0];
				this._doubleLastComboLow = Low[0];
				if (this._intComboCount == 13)
				{
					if (this._intSequentialRecycleCount >= 18) // (page 22 of Bloomberg DeMark Indicators book)
					{
						this.RecycleComboCountdown();
					}
					else
					{
						if (this.TDCountdownType == TDType.TDBuy && this.DrawBar13Arrow) DrawArrowUp(this._intArrowCount++.ToString(), false, 0, this._doubleYLocation - this._doublePointOffset * 1, Color.Lime);
						if (this.TDCountdownType == TDType.TDSell && this.DrawBar13Arrow) DrawArrowDown(this._intArrowCount++.ToString(), false, 0, this._doubleYLocation + this._doublePointOffset * 1, Color.Red);
						this.InitializeNewComboCountdown();
					}
				}
			}
			else if ((this.TDCountdownType == TDType.TDBuy && (Low[0] > this._doubleSetupCountdownBar1High || this.IsBarTDSellSetup)) || 
				(this.TDCountdownType == TDType.TDSell && (High[0] < this._doubleSetupCountdownBar1Low || this.IsBarTDBuySetup)))
			{
				this.RemoveCurrentComboCountdown();
				this.InitializeNewComboCountdown();
			}
		}
		
		private void RecycleComboCountdown()
		{
			this.RemoveCurrentComboCountdown();
			this.TDComboDrawText("R", 0, this._doubleYLocation, this._colorComboFontColorRecycle, this._intComboFontSizeRecycle);
			if (this.TDCountdownType == TDType.TDBuy)
				DrawArrowUp(this._intArrowCount++.ToString(), false, 0, this._doubleYLocation - this._doublePointOffset * 1, Color.Black);
			else
				DrawArrowDown(this._intArrowCount++.ToString(), false, 0, this._doubleYLocation + this._doublePointOffset * 1, Color.Black);
			this._intComboCount = 0;
			this._strComboTags.Initialize();
		}

		private void RemoveCurrentComboCountdown()
		{
			for (int int1 = 0; int1 < this._intComboCount; int1++) RemoveDrawObject(this._strComboTags[int1]);
		}

		private void StartComboCountDown()
		{
			for (int index = 8; index >= 1; index--) 
			{
				if (this.TDCountdownType == TDType.TDBuy)
				{
					if ((index == 8 && Close[index] <= Low[index+2] && Close[index] < Close[index+1]) ||
						(Close[index] <= Low[index+2] && Low[index] <= this._doubleLastComboLow && Close[index] < this._doubleLastComboClose && Close[index] < Close[index+1]))
					{
						this._dsYLocation.Set(index, this._dsYLocation[index] + this._doublePointOffset * this._doubleMult);
						this._intComboCount++;
						this.TDComboDrawText(this._intComboCount.ToString(), index,	this._dsYLocation[index], Color.White, this._intComboFontSize);
						this._doubleLastComboClose = Close[index];
						this._doubleLastComboLow = Low[index];
					}
				}
				else
				{
					if ((index == 8 && Close[index] >= High[index+2] && Close[index] > Close[index+1]) ||
						(Close[index] >= High[index+2] && High[index] >= this._doubleLastComboHigh && Close[index] > this._doubleLastComboClose && Close[index] > Close[index+1]))
					{
						this._dsYLocation.Set(index, this._dsYLocation[index] + this._doublePointOffset * this._doubleMult);
						this._intComboCount++;
						this.TDComboDrawText(this._intComboCount.ToString(), index,	this._dsYLocation[index], Color.White, this._intComboFontSize);
						this._doubleLastComboClose = Close[index];
						this._doubleLastComboHigh = High[index];
					}
				}
			}
			this.IsComboCountDownInProgress = true;
		}
		
		private void TDComboDrawText(string strText, int intBarsAgo, double	doubleYLocation, Color colorColor, int intFontSize)
		{
			this._intObjectCount++;
			DrawText(this._intObjectCount.ToString(), 
				false, 
				strText, 
				intBarsAgo, 
				doubleYLocation, 
				colorColor, 
				new Font(FontFamily.GenericSansSerif, intFontSize, FontStyle.Bold, GraphicsUnit.Pixel), 
				StringAlignment.Center, 
				Color.Transparent, 
				Color.Transparent, 
				0); 
			this._strComboTags.SetValue(this._intObjectCount.ToString(), _intComboCount-1); 
		}
		#endregion
		
		#endregion
		
		#region Private Properties
        private bool BearishTDPriceFlipOccurred
        {
            get { return this._boolBearishTDPriceFlipOccurred; }
			set { this._boolBearishTDPriceFlipOccurred = value; }
        }
        private bool BullishTDPriceFlipOccurred
        {
            get { return this._boolBullishTDPriceFlipOccurred; }
			set { this._boolBullishTDPriceFlipOccurred = value; }
        }
		/// <summary>
		/// (page 2 of Bloomberg DeMark Indicators book)
		/// </summary>
		/// <returns></returns>
        private bool IsBearishFlip
        {
            get { return Close[0] < Close[4] && Close[1] > Close[5]; }
        }
		/// <summary>
		/// (page 4 of Bloomberg DeMark Indicators book)
		/// </summary>
		/// <returns></returns>
        private bool IsBullishFlip
        {
            get { return Close[0] > Close[4] && Close[1] < Close[5]; }
        }
        private bool IsComboCountDownCompleted
        {
            get { return this._intComboCount == 13; }
        }
        private bool IsComboCountDownInProgress
        {
            get { return this._boolComboCountownInProgress; }
			set { this._boolComboCountownInProgress = value; }
        }
        private bool IsSequentialCountdownInProgress
        {
            get { return this._boolSequentialCountownInProgress; }
			set { this._boolSequentialCountownInProgress = value; }
        }
        private bool IsSetupCountDownCompleted
        {
            get { return this._intSetupCount == this._intSetupBars; }
        }
        private bool IsSetupCountdownInProgress
        {
            get { return this._intSetupCount > 0 && this._intSetupCount < this._intSetupBars; }
        }
		/// <summary>
		/// (page 3 of Bloomberg DeMark Indicators book)
		/// </summary>
		/// <returns></returns>
		private bool IsBarTDBuySetup
		{
			get
			{
				return Close[0] < Close[4] &&
					Close[1] < Close[5] &&
					Close[2] < Close[6] &&
					Close[3] < Close[7] &&
					Close[4] < Close[8] &&
					Close[5] < Close[9] &&
					Close[6] < Close[10] &&
					Close[7] < Close[11] &&
					Close[8] < Close[12];
			}
		}
		/// <summary>
		/// (page 4 of Bloomberg DeMark Indicators book)
		/// </summary>
		/// <returns></returns>
		private bool IsBarTDSellSetup
		{
			get
			{
				return Close[0] > Close[4] &&
					Close[1] > Close[5] &&
					Close[2] > Close[6] &&
					Close[3] > Close[7] &&
					Close[4] > Close[8] &&
					Close[5] > Close[9] &&
					Close[6] > Close[10] &&
					Close[7] > Close[11] &&
					Close[8] > Close[12];
			}
		}
		#endregion
		
		#region Public Properties
		
		#region Parameters
        [Description("Draw bar 13 arrow.")]
        [Category("1-Parameters")]
		[Gui.Design.DisplayName("Draw bar 13 arrow")]
        public bool DrawBar13Arrow
        {
            get { return this._boolDrawBar13Arrow; }
            set { this._boolDrawBar13Arrow = value; }
        }		

        [Description("Draw price flip arrow.")]
        [Category("1-Parameters")]
		[Gui.Design.DisplayName("Draw price flip arrow")]
        public bool DrawPriceFlipArrow
        {
            get { return this._boolDrawPriceFlipArrow; }
            set { this._boolDrawPriceFlipArrow = value; }
        }		

        [Description("Plot Combo Countdown.")]
        [Category("1-Parameters")]
		[Gui.Design.DisplayName("Plot Combo Countdown")]
        public bool PlotComboCountDown
        {
            get { return this._boolPlotComboCountDown; }
            set { this._boolPlotComboCountDown = value; }
        }		

        [Description("Plot Sequential Countdown.")]
        [Category("1-Parameters")]
		[Gui.Design.DisplayName("Plot Sequential Countdown")]
        public bool PlotSequentialCountDown
        {
            get { return this._boolPlotSequentialCountDown; }
            set { this._boolPlotSequentialCountDown = value; }
        }		

        [Description("TD Countdown type.")]
        [Category("1-Parameters")]
		[Gui.Design.DisplayName("TD Countdown type")]
        public TDType TDCountdownType
        {
            get { return this._TDCountdownType; }
            set { this._TDCountdownType = value; }
        }		
		#endregion
		
		#region Sequential
		[XmlIgnore()]
        [Description("Font color.")]
        [Category("4-Sequential Countdown Parameters")]
		[Gui.Design.DisplayName("Font color")]
        public Color SequentialFontColor
        {
            get { return _colorSequentialFontColor; }
            set { _colorSequentialFontColor = value; }
        }
		[Browsable(false)]
		public string SequentialFontColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorSequentialFontColor); }
			set { _colorSequentialFontColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

        [Description("Font size.")]
        [Category("4-Sequential Countdown Parameters")]
		[Gui.Design.DisplayName("Font size")]
        public int SequentialFontSize
        {
            get { return _intSequentialFontSize; }
            set { _intSequentialFontSize = Math.Min(Math.Max(5, value), 21); }
        }
		#endregion
		
		#region Combo
		[XmlIgnore()]
        [Description("Font color.")]
        [Category("5-Combo Countdown Parameters")]
		[Gui.Design.DisplayName("Font color")]
        public Color ComboFontColor
        {
            get { return _colorComboFontColor; }
            set { _colorComboFontColor = value; }
        }
		[Browsable(false)]
		public string ComboFontColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorComboFontColor); }
			set { _colorComboFontColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

        [Description("Font size.")]
        [Category("5-Combo Countdown Parameters")]
		[Gui.Design.DisplayName("Font size")]
        public int ComboFontSize
        {
            get { return _intComboFontSize; }
            set { _intComboFontSize = Math.Min(Math.Max(5, value), 21); }
        }
		#endregion
		
		#region Setup
		[XmlIgnore()]
        [Description("Font color.")]
        [Category("2-Setup Countdown Parameters")]
		[Gui.Design.DisplayName("Font color")]
        public Color SetupFontColor
        {
            get { return _colorSetupFontColor; }
            set { _colorSetupFontColor = value; }
        }
		[Browsable(false)]
		public string SetupFontColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorSetupFontColor); }
			set { _colorSetupFontColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
        [Description("Font color bar 1.")]
        [Category("2-Setup Countdown Parameters")]
		[Gui.Design.DisplayName("Font color bar 1")]
        public Color SetupFontColorBar1
        {
            get { return _colorSetupFontColorBar1; }
            set { _colorSetupFontColorBar1 = value; }
        }
		[Browsable(false)]
		public string SetupFontColorBar1Serialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorSetupFontColorBar1); }
			set { _colorSetupFontColorBar1 = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		[XmlIgnore()]
        [Description("Font color bar 9.")]
        [Category("2-Setup Countdown Parameters")]
		[Gui.Design.DisplayName("Font color bar 9")]
        public Color SetupFontColorBar9
        {
            get { return _colorSetupFontColorBar9; }
            set { _colorSetupFontColorBar9 = value; }
        }
		[Browsable(false)]
		public string SetupFontColorBar9Serialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorSetupFontColorBar9); }
			set { _colorSetupFontColorBar9 = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		[XmlIgnore()]
        [Description("Font color price flip.")]
        [Category("2-Setup Countdown Parameters")]
		[Gui.Design.DisplayName("Font color price flip")]
        public Color SetupFontColorPriceFlip
        {
            get { return _colorSetupFontColorPriceFlip; }
            set { _colorSetupFontColorPriceFlip = value; }
        }
		[Browsable(false)]
		public string SetupFontColorPriceFlipSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorSetupFontColorPriceFlip); }
			set { _colorSetupFontColorPriceFlip = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

        [Description("Font size.")]
        [Category("2-Setup Countdown Parameters")]
		[Gui.Design.DisplayName("Font size")]
        public int SetupFontSize
        {
            get { return _intSetupFontSize; }
            set { _intSetupFontSize = Math.Min(Math.Max(5, value), 21); }
        }

        [Description("Setup look back bars.")]
        [Category("2-Setup Countdown Parameters")]
		[Gui.Design.DisplayName("Setup look back bars")]
        public int SetupLookBackBars
        {
            get { return _intSetupLookBackBars; }
            set { _intSetupLookBackBars = Math.Max(4, value); }
        }

        [Description("Number of bars needed for Setup.")]
        [Category("2-Setup Countdown Parameters")]
		[Gui.Design.DisplayName("Number of bars needed for Setup")]
        public int SetupBars
        {
            get { return _intSetupBars; }
            set { _intSetupBars = Math.Max(9, value); }
        }

        [Description("Plot type.")]
        [Category("2-Setup Countdown Parameters")]
		[Gui.Design.DisplayName("Plot type")]
        public TDSequentialPlotType PlotType
        {
            get { return _TDSequentialPlotType; }
            set { _TDSequentialPlotType = value; }
        }

        [Description("Draw bar 1 arrow.")]
        [Category("2-Setup Countdown Parameters")]
		[Gui.Design.DisplayName("Draw bar 1 arrow")]
        public bool DrawBar1Arrow
        {
            get { return this._boolDrawBar1Arrow; }
            set { this._boolDrawBar1Arrow = value; }
        }

        [Description("Draw perfect signal arrow.")]
        [Category("2-Setup Countdown Parameters")]
		[Gui.Design.DisplayName("Draw perfect signal arrow")]
        public bool DrawPerfectSignalArrow
        {
            get { return this._boolDrawPerfectSignalArrow; }
            set { this._boolDrawPerfectSignalArrow = value; }
        }
		#endregion
		
		#region Setup TDST
		[XmlIgnore()]
        [Description("Buy Color.")]
        [Category("3-Setup TDST Parameters")]
		[Gui.Design.DisplayName("Buy Color")]
        public Color TDSTBuyColor
        {
            get { return _colorSetupTDSTBuy; }
            set { _colorSetupTDSTBuy = value; }
        }
		[Browsable(false)]
		public string TDSTBuyColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorSetupTDSTBuy); }
			set { _colorSetupTDSTBuy = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		[XmlIgnore()]
        [Description("Dash Style.")]
        [Category("3-Setup TDST Parameters")]
		[Gui.Design.DisplayName("Dash Style")]
        public DashStyle TDSTDashStyle
        {
            get { return _dsTDST; }
            set { _dsTDST = value; }
        }

        [Description("Length (9-610).")]
        [Category("3-Setup TDST Parameters")]
		[Gui.Design.DisplayName("Length")]
        public int TDSTLength
        {
            get { return _intSetupTDSTLength; }
            set { _intSetupTDSTLength = Math.Min(Math.Max(9, value), 610); }
        }

        [Description("Maximun Number on Chart (1-13).")]
        [Category("3-Setup TDST Parameters")]
		[Gui.Design.DisplayName("Maximun Number on Chart")]
        public int TDSTMaxNumber
        {
            get { return _intSetupTDSTMaxNumber; }
            set { _intSetupTDSTMaxNumber = Math.Min(Math.Max(1, value), 13); }
        }

        [Description("TDST On.")]
        [Category("3-Setup TDST Parameters")]
		[Gui.Design.DisplayName("TDST On")]
        public bool TDSTOn
        {
            get { return this._boolSetupTDSTOn; }
            set { this._boolSetupTDSTOn = value; }
        }

		[XmlIgnore()]
        [Description("Sell Color.")]
        [Category("3-Setup TDST Parameters")]
		[Gui.Design.DisplayName("Sell Color")]
        public Color TDSTSellColor
        {
            get { return _colorSetupTDSTSell; }
            set { _colorSetupTDSTSell = value; }
        }
		[Browsable(false)]
		public string TDSTSellColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorSetupTDSTSell); }
			set { _colorSetupTDSTSell = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

        [Description("Width (0-13).")]
        [Category("3-Setup TDST Parameters")]
		[Gui.Design.DisplayName("Width")]
        public int TDSTWidth
        {
            get { return _intSetupTDSTWidth; }
            set { _intSetupTDSTWidth = Math.Min(Math.Max(0, value), 13); }
        }
		#endregion
		
		#endregion
    }
}

public enum TDSequentialPlotType
{
	DrawDots,
	DrawText
}

public enum TDType
{
	TDBuy,
	TDSell
}

public struct TDSTHelper 
{
	public string TDSTTag;
	public int TDSTStartBar;
	public double TDSTStartY;
	public int TDSTEndBar;
	public double TDSTEndY;
	public Color TDSTColor;
	public DashStyle TDSTDashStyle;
	public int TDSTWidth;
	
	public TDSTHelper(string strTag, int intStartBar, double doubleStartY, int intEndBar, double doubleEndY, Color colorTDST, DashStyle dsTDST, int intWidth) 
	{
		TDSTTag = strTag;
		TDSTStartBar = intStartBar;    
		TDSTStartY = doubleStartY;    
		TDSTEndBar = intEndBar;    
		TDSTEndY = doubleEndY;    
		TDSTColor = colorTDST;    
		TDSTDashStyle = dsTDST;    
		TDSTWidth = intWidth;    
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private IGTDSequential[] cacheIGTDSequential = null;

        private static IGTDSequential checkIGTDSequential = new IGTDSequential();

        /// <summary>
        /// TD Sequential/Combo Countdown
        /// </summary>
        /// <returns></returns>
        public IGTDSequential IGTDSequential()
        {
            return IGTDSequential(Input);
        }

        /// <summary>
        /// TD Sequential/Combo Countdown
        /// </summary>
        /// <returns></returns>
        public IGTDSequential IGTDSequential(Data.IDataSeries input)
        {

            if (cacheIGTDSequential != null)
                for (int idx = 0; idx < cacheIGTDSequential.Length; idx++)
                    if (cacheIGTDSequential[idx].EqualsInput(input))
                        return cacheIGTDSequential[idx];

            IGTDSequential indicator = new IGTDSequential();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.SetUp();

            IGTDSequential[] tmp = new IGTDSequential[cacheIGTDSequential == null ? 1 : cacheIGTDSequential.Length + 1];
            if (cacheIGTDSequential != null)
                cacheIGTDSequential.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheIGTDSequential = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// TD Sequential/Combo Countdown
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.IGTDSequential IGTDSequential()
        {
            return _indicator.IGTDSequential(Input);
        }

        /// <summary>
        /// TD Sequential/Combo Countdown
        /// </summary>
        /// <returns></returns>
        public Indicator.IGTDSequential IGTDSequential(Data.IDataSeries input)
        {
            return _indicator.IGTDSequential(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// TD Sequential/Combo Countdown
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.IGTDSequential IGTDSequential()
        {
            return _indicator.IGTDSequential(Input);
        }

        /// <summary>
        /// TD Sequential/Combo Countdown
        /// </summary>
        /// <returns></returns>
        public Indicator.IGTDSequential IGTDSequential(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.IGTDSequential(input);
        }

    }
}
#endregion
